'
' Step 3: Data transfer, Spin to assembly, hub to cog.
'
' In this tutorial functions are created in the object that load the hub ram with the on time and off time
' for a flashing led.
'
' The assembly code then uses rdlong to get this information to use.

' What happens in words:

' 1. The start function is called from the demo program this firstly loads the HUB ram with intial values for the pins and
'    the on and off times then it calls coginit to start the assembly cog.
'
' 2. The address of the first hub variable used is passed to the assembly using the coginit command, the @ symbol means the
'    address of the variable.
'
' 3. In the assembly the par variable automatically contains the pointer to the hub variables
'
' 4. The rdlong command is used to read from that address into a local COG variable so it can be used in the assembly.
'
' 5. pins are set up and the loop starts, it simply flashes the led according to the on_time and off_time that is in HUB ram
'
' 6. At the end of each loop the variables are reloaded from the hub as they may have changed.
'
' 7. Spin functions in the demo program are used to change the hub ram variables, they are helper functions and will
'    actually reside in the cog where they are caled rather than the assembly one.
'
'  Note: the pins are also held in hub ram so you could make functions to change the pin assignments too, but you will need
'        to alter the assembly to make use of this
'
       
VAR

  long  cog                           ' This is in hub ram
  long  pin1
  long  pin2
  long  on_time
  long  off_time

PUB start(p1,p2,on,off) : okay  

pin1     := |< p1          ' Put in HUB ram.      
pin2     := |< p2
on_time  := on             ' the initial values used
off_time := off

okay := cog := cognew(@entry, @pin1) + 1    '** NOTE: we send the cog the address to the hub ram variables used **  

PUB stop

'' Stop flashing - frees a cog

  if cog
    cogstop(cog~ -  1) 

PUB setontime(on)                           ' Used by spin programs to set on_time

on_time := on

PUB setofftime(off)

off_time := off

DAT

entry
                      ' When the cog starts par is initialized with the address passed in the coginit command
                      ' in this case it was passed @pin1
                      
                      mov     p,par           ' load parameter pointer into p (points to global variables)
                                              ' p is just a copy and keeps par unchanged for later. 
                      rdlong  _pin1,p         ' load first parameter into _pin1 (cog ram)
                      add     p,#4            ' Increment address to next global variable, the address is in bytes so
                                              ' it needs 4 adding to get to the next long
                      rdlong  _pin2,p         ' Load second parameter pin2 into _pin2
                      add     p,#4            
                      rdlong  _on_time,p      ' get the on time
                      add     p,#4            
                      rdlong  _off_time,p     ' and the off

                      ' Let us set up a bit mask (see part 2)
                      mov     _pinmask,_pin1
                      or      _pinmask,_pin2
                      ' Set the pins as outputs
                      or     dira, _pinmask
                      ' Make pins high
                      or     outa, _pinmask

                      mov    time,cnt         ' find the current time
                      add    time,_on_time    ' create target time

:loop                 waitcnt time,_off_time  ' wait for delay to pass (remember off_time is added to time by the command)
                      xor     outa ,_pinmask  ' toggle the pins
                      waitcnt time,_on_time   ' wait for delay to pass
                      xor     outa,_pinmask   ' toggle
                      
                      ' reload the variables from hub ram as they may have been changed
                      mov     p,par
                      add     p,#8            ' we add 8 to skip the first two variables (pin1 and pin2)                                           
                      rdlong  _on_time, p     ' load on time   
                      add     p,#4                                              
                      rdlong  _off_time,p     ' and off time

                      jmp    #:loop           ' and loop away
                                                

                       ' These variables are in cog ram!                 
' Uninitialized data

time                 res     1
_pinmask             res     1
_pin1                res     1
_pin2                res     1
_on_time             res     1
_off_time            res     1
p                    res     1