' Step 2
' Setting pins:
' Use various methods to set pins high and low, introduces the idea of masks and how bitwise logic can
' be used to alter a register selectively.
' Shows use of muxz and muxnz for setting and clearing bits as an alternative.

'****************************************************************************************************
' Aside: bit masks

' Remember your truth tables:

' OR    A  B   Result      XOR   A    B   Result         ANDN    A     B    B!   Result (A and B!)
'       0  0     0               0    0     0                    0     0    1     0
'       0  1     1               0    1     1                    0     1    0     0
'       1  0     1               1    0     1                    1     0    1     1
'       1  1     1               1    1     0                    1     1    0     0


' e.g.    or    outa,mask
'         xor   outa,mask
'         andn  outa,mask

'  OR  : Any bits set high in mask will be set high in outa
' XOR  : Any bits set high in the mask will be toggled in the outa
' ANDN : Any bits set high in the mask will be set low in the outa 
'***************************************************************************************************

       
VAR

  long  cog                           ' This is in hub ram

PUB start(pin1,pin2,pin3,pin4) : okay

_pin1   := pin1             ' Loaded when cog is loaded (see step 1)       
_pin2   := pin2
_pin3   := pin3
_pin4   := pin4

okay := cog := cognew(@entry, 0) + 1  


PUB stop

'' Stop flashing - frees a cog

  if cog
    cogstop(cog~ -  1) 


DAT

entry                  

                      ' Start by setting the DIRA register

                      mov     pinmask1, #1      ' pinmask is loaded with 1, i.e ...00001
                      shl     pinmask1, _pin1   ' shifted left, if pin1 = 3  would be ..001000
                      or      dira, pinmask1    ' OR with dira will set the correct bit high but not affect others
                      
                      ' Repeat for other pin
                      mov     pinmask2, #1
                      shl     pinmask2, _pin2
                      or      dira, pinmask2    'The pin masks may also be used for changing the state of the output

                      ' And here is a mask to control two pins, 3 and 4
                      mov     pinmask3_4,#1       ' load mask with one
                      shl     pinmask3_4, _pin3   ' shift to set bit for pin 3
                      mov     t,#1                ' load a temporary variable with one
                      shl     t,_pin4             ' shift to set bit for pin 4
                      or      pinmask3_4, t       ' put in pinmask with pin 3
                      or      dira,pinmask3_4     ' and modify dira to match
                      or      outa,pinmask3_4     ' set the pins high                       

                      mov     time,cnt              ' Current clock loaded into time
                      add     time,delay            ' delay added to time


                      
                      ' make pin 1 high 
:loop                 or      outa, pinmask1
                      ' make pin 2 low
                      andn    outa, pinmask2

                      waitcnt time,delay            ' wait until cnt=time then add delay to time

                      ' make pin 1 low
                      andn    outa, pinmask1
                      ' make pin 2 high
                      or      outa, pinmask2       

                      waitcnt time,delay            ' wait a short time

                      ' toggle pin1
                      xor     outa, pinmask1
                      ' toggle pin2
                      xor     outa, pinmask2        ' In this case also toggles

                      waitcnt time,delay

                      ' Now to use muxz and muxnz,  another way to set/clear pins

                      mov     t, #1         wz      ' all this does is set the zero flag
                      ' make pin1 low
                      muxz    outa,pinmask1                     
                      ' make pin2 high
                      muxnz   outa,pinmask2

                      waitcnt time,delay

                      ' toggle pins 3 ans 4 together
                      xor     outa,pinmask3_4

                      jmp     #:loop

                      'And do it all again!
                             

                       ' These variables are in cog ram!
                        
' Initialized data                                     

delay                long    10000000                   ' delay in clock cycles
_pin1                long    0
_pin2                long    0
_pin3                long    0
_pin4                long    0                      

'**** ATTENTION, all res variables MUST come after other variables ************

' Uninitialized data

time                 res     1
pinmask1             res     1
pinmask2             res     1
pinmask3_4           res     1
t                    res     1
