'' ================================================================================================
''
''   File....... jw_bicolor_v3_demo2.spin
''   Purpose.... bi-color LED driver demo with Spin fader object in second cog 
''   Author..... Jon Williams
''   E-mail..... jwilliams@efx-tek.com
''   Started.... 
''   Updated.... 13 MAY 2009 
''
'' ================================================================================================


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  
  MS_001 = 80_000_000 / 1_000                                   ' ticks per millisecond
  
  
OBJ

  led : "jw_bicolor_v3"

  
VAR

  long  brPntr                                                  ' pointer to led brightness control
  long  fadeStart                                               ' starting level of fade
  long  fadeStop                                                ' ending level of fade
  long  fadeDelay                                               ' fade step timing (ms)
  long  fadeActive                                              ' fader control (true or false)
  
  long  stack[16]                                               ' stack for spin object


PUB main | okay, idx

  okay := led.start(8, led#LED_OFF, 100, led#LED_OFF, 100, 100)
  if okay
    brPntr := led.getbrightpntr                                 ' get pointer to brightness control
    cognew(fader, @stack)                                       ' launch fader into own cog

  repeat                                                        ' loop forever
    repeat idx from led#LED_OFF to led#LED_RED                  ' cycle through LED colors
      led.setcolor(idx)
      waitcnt(MS_001 * 500 + cnt)                               ' hold 0.5 seconds
      
    led.off
    waitcnt(MS_001 * 250 + cnt)
    
    led.red 
    fade(0, 100, 20)                                            ' start background fade
    waitcnt(MS_001 * 3_000 + cnt)                               ' wait during fade


PUB fade(s, p, d)

'' Initiates automatitic fade of led object
'' -- s = starting level for fade
'' -- p = ending level for fade
'' -- d = delay per step (milliseconds)

  long[brPntr] := s                                             ' set LED to starting brightness
  fadeStart := s                                                ' set fader parameters
  fadeStop := p 
  fadeDelay := d
  fadeActive := true


PUB nofade(b)

'' Stops background fader
'' b = level for led brightness (0 to 100)

  long[brPntr] := b                                             ' set LED to starting brightness
  fadeStart := b                                                ' equalize levels to stop fade
  fadeStop := b
  fadeActive := false                                           ' disable fader

    
PRI fader

'' Background fader control
'' -- must be launched into own cog

  fadeActive := false                                           ' start disabled

  repeat
    if fadeActive                                               ' fader running?
      long[brPntr] := fadeStart                                 ' update brightness 
      if fadeDelay > 0                                          ' delay specified?
        waitcnt(fadeDelay * MS_001 + cnt)                       ' yes, hold
      if fadeStart < fadeStop                                   ' below target?
        ++fadeStart                                             ' yes, increment        
      elseif fadeStart > fadeStop                               ' above target?
        --fadeStop                                              ' yes, decrement
      else
        fadeActive := false                                     ' at target, disable fader
    
 