{*******************************************************
* (C) 2010 HannoWare.com                               *
* Drive 2 motors with h-bridge and read encoder        *
* AppletImage=motor.gif                                *
********************************************************}
CON
  _clkmode        = xtal1 + pll16x
  _xinfreq        = 5_000_000
  MAXSPEED=4000
  motorUpdateHz=200
  'pins for h-bridge: 3,4,26,27
  'pins for encoder: 12,13
  m1Dir= |<3
  m1Pwm= |<4
  m2Dir= |<26
  m2Pwm= |<27
var
  long pwm[6]  'structure populated by spin code to set motor direction
obj
  vp : "Conduit"
  qs:"QuickSample"
pub Main|mvel,pos[2],cmd[2],join,but,frame[400],lastP,ncnt
  {testing harness for driving/measuring motors
    mvel=motor velocity- only monitoring one encoder
    pos=position for two motors
    cmd=command power for two motors
    join=set to 1 by viewport to tell both motors to move at same speed
    but=button clicked in viewport to trigger a speed test
    frame=used by quicksample to measure the io port
    lastp= last position- used to calculate mvel
    ncnt= next cnt, used to control time of loop}
  
  vp.register(qs.sampleINA(@frame,1)) 'sample INA with 1 cog up to 20Mhz
  vp.config(string("var:io(bits=[4m1p,3m1r,27m2p,26m2r,12enc,13enc]),mvel,pos1,pos2,m1(min=-5000,max=5000),m2(min=-5000,max=5000),join,but"))
  vp.config(string("lsa:view=io,timescale=1ms,trigger=io[30]r")) 
  vp.config(string("dso:view=[mvel,pos1,pos2,m1,m2],timescale=1s"))
  vp.config(string("edit:join(mode=switch),m1(mode=scroll),m2(mode=scroll)"))
  vp.config(string("start:dso"))
  vp.share(@mvel,@but)
  start(@pos)
  ncnt:=cnt
  if 0 'set to 1 to do speed test
    repeat
      if but
        ncnt:=cnt
        pos~
        lastP~
        repeat motorUpdatehz
          Update(cmd[0],cmd[0])
          waitcnt(ncnt+=clkfreq/motorUpdateHz)
          mvel:=pos-lastP
          lastP:=pos
        Update(0,0)        
        but~ 
  repeat
    if join
      Update(cmd[0],cmd[0])
    else
      Update(cmd[0],cmd[1])      
    mvel:=pos-lastP
    lastP:=pos
    waitcnt(ncnt+=clkfreq/motorUpdateHz)'}}  

pub Start(posPtr)
'starts 1 encoder cog and 1 motor cog.  initializes positions
  setupEncoder(posPtr)
  setupMotor
  long[posPtr]~
  long[posPtr+1]~

pub Update(speed,speed2)|p0,p2,t
'updates motor control with desired speed
  p0:=m1pwm+m2pwm
  if speed>0
    p0+=m1dir
  if speed2>0
    p0+=m2dir
  pwm[0]:=p0    
  speed:= (||speed) <#MAXSPEED
  speed2:= (||speed2) <#MAXSPEED
  if speed > speed2
    p0-=m2pwm
    pwm[1]:=(speed2)+5  '2505
    pwm[3]:=(speed-speed2)+5 '2505
    pwm[5]:=5+(5000-speed)   '15 
  else
    p0-=m1pwm
    pwm[1]:=(speed)+5
    pwm[3]:=(speed2-speed)+5
    pwm[5]:=5+(5000-speed2)
  pwm[2]:=p0
  pwm[4]:=0   
  
pub setupMotor
  pwm[1]:=5
  pwm[3]:=5
  pwm[5]:=5
  diraV:=m1Dir|m1Pwm|m2Dir|m2Pwm
  andV:=!diraV
  cognew(@doPWM,@pwm)
  
pub setupEncoder(posPtr)
  cognew(@count,posPtr)

dat
        org
count
        mov m2w,par
        add m2w,#4 
        mov  :t,ina              'get ina- 1011
:loop  ' 21 instr= 80 cnts=1Ms/sec
        and :t,mAEnc             't shows before flank, what A bit where
        waitpne :t,mAEnc         'wait for encoder A bits to change
        mov :t1,ina                
        and :t1,mABEnc           't1 shows after flank, what AB bits are 
        mov :t2,:t1
        and :t2,mAEnc
        xor :t2,:t               ' t2 shows which A bits changed
        mov :t,:t1               ' get t ready for next measurement
        shr :t2,#12
        rcr :t2,#1   wc          ' check if A on motor 1 changed        
  if_nc shr :t1,#14               ' leave both t2, t1 shifted if take jump
  if_nc jmp #:doM2                '
        shr :t1,#12
        rcr :t1,#1   wc          ' check if A=1
  if_nc shr :t1,#1
  if_nc jmp #:doM2      
        rcr :t1,#1   wc          ' check fwd/bkw on motor 1
        rdlong m1Cnt,par'added
  if_c add m1Cnt,#1
  if_nc  sub m1Cnt,#1
        wrlong m1Cnt,par         
        'ready to do motor 2, t1 shifted 14, t2 shifted 13
:doM2
        shr :t2,#9
        rcr :t2,#1   wc          ' check if A on motor 2 changed
  if_nc jmp #:loop
        shr :t1,#8
        rcr :t1,#1   wc          ' check if a=1
  if_nc jmp #:loop
        rcr :t1,#1    wc          ' check if fwd/bkw on motor 2
        rdlong m2cnt,m2w   'added
  if_nc add m2Cnt,#1
  if_c  sub m2Cnt,#1         
        wrlong m2Cnt,m2w               
        jmp #:loop
:t2     long 1      
:t1     long 1        
:t      long 1
m2W     long 1
mAEnc   long %0100_0000_0001_0000_0000_0000 '12
mABEnc  long %1100_0000_0011_0000_0000_0000' 12,13 '00011
m1Cnt   long 0
m2Cnt   long 0
' outputs pwm to motors
' uses pwm structure to get bit values and duty cycles
        org
doPWM   or dira,dirav
 loop   mov ptr,par      
        mov :i,#3             
:duty   rdlong orV,ptr   'get value to or
        add ptr,#4
        mov :t,outa
        rdlong timeV,ptr  'get wait
        add ptr,#4
        and :t,andV
        or :t,orV
        mov outa,:t
        add timeV,cnt
        waitcnt timeV,#0
        djnz :i,#:duty
        jmp #loop
:t      long 1
:i      long 1        
diraV   long 1
andV    long 1      
ptr     long 1
orV     long 1        
timeV   long 1
    
            