''*****************************************************
''* Squish Kevin game.
''* Written by: Shane Avery
''*****************************************************

CON 
  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000 + 0000
  _stack = ($3000 + 64) >> 2 

  ' graphics driver and screen constants
  PARAMCOUNT = 14
  ONSCREEN_BUFFER = $5000

  ' size of the graphics tile map
  X_TILES = 16
  Y_TILES = 12

  SCREEN_WIDTH = 256
  SCREEN_HEIGHT = 192

  'Color constants 
  COL_Black     = %0000_0010
  COL_Blue      = %0001_1_100
  COL_Green     = %0101_1_100
  COL_Red       = %1010_1_100
  COL_White     = %0000_0111

  COLOR_0 = (COL_Black << 0)
  COLOR_1 = (COL_Blue << 8)
  COLOR_2 = (COL_Red << 16)
  COLOR_3 = (COL_White << 24)

  ' NES bit encodings for NES gamepad 0
  NES0_RIGHT  = %00000000_00000001
  NES0_LEFT   = %00000000_00000010
  NES0_DOWN   = %00000000_00000100
  NES0_UP     = %00000000_00001000
  NES0_START  = %00000000_00010000
  NES0_SELECT = %00000000_00100000
  NES0_B      = %00000000_01000000
  NES0_A      = %00000000_10000000

VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only                            

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  byte  dip[4]        'destination ip. the ip address we commune with          
  byte  subnet[4]     'subnet
  byte  ip[4]         'our ip address
  byte  gateway[4]    'gateway ip address
  byte  mac[8]        'our mac address

  long  cog_stack[256] 'stack space for the new cogs

  long  winloss       'variable to indicate if we have won or lost
  long  player_score  'variable to indicate the current player score
  long  opp_score     'variable to indicate the opponent score
  long  score_change  'variable to indicate to the main display
                      'loop that a score has changed

  byte  eth_lock      'lock to manage the EtherX between cogs                       

OBJ

  tv    : "tv_drv_010.spin"
  gr    : "graphics_drv_010"
  pad   : "gamepad_drv_001"
  eth   : "W5100_drv_011"

DAT                           
tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    60_000_000'_xinfreq<<4  'broadcast
                        long    0               'auralcog

PUB start | i, dx, dy

  '///////////////////////////////////////////////////////////
  'Begin the initialization and display Splash screen
  '///////////////////////////////////////////////////////////
  
  'start gamepad
  pad.start
  
  'start the W5100 driver
  eth.start

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors, each tile has same 4 colors
  repeat i from 0 to 64
    colors[i] := COLOR_3 | COLOR_2 | COLOR_1 | COLOR_0

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := ONSCREEN_BUFFER >> 6 + dy + dx * tv_vc + ((dy&$3f)<<10)

  'start and setup the graphics driver
  gr.start
  gr.setup(X_TILES, Y_TILES, 0, 0, ONSCREEN_BUFFER)

  'display the splash screen for the game
  gr.clear
  gr.colorwidth(2,2)
  gr.textmode(2,2,6,0)
  gr.text(50,150,@title)
  waitcnt(cnt + $bebc200)      'we will wait a couple seconds for the ethernet to come up

  'Now that the ethernet is up the user can be prompted
  'to push start to begin the same
  gr.colorwidth(1,0)
  gr.textmode(1,1,6,0)
  gr.text(90,130,@pushst)

  repeat while pad.read & NES0_START == 0

  'Initialize the ethernet device
  InitEther

  'Initialize game variables
  winloss := 0
  player_score := 0
  opp_score := 0
  score_change := 0

  'Create a new lock to manage the EtherX card between cogs
  eth_lock := locknew

  'Start the gamedpad monitor in a new COG
  cognew(MonitorButtons, @cog_stack[0])

  'Start the ethernet monitor in a new COG
  cognew(MonitorEthernet, @cog_stack[128])

  '//////////////////////////////////////////////////////////
  'Begin the game loop
  '////////////////////////////////////////////////////////// 
  'Draw the playing field.
  gr.clear
  
  gr.colorwidth(3,3)
  gr.textmode(6,6,6,0)
  gr.text(50,60,@pscore)
  
  gr.colorwidth(3,3)
  gr.textmode(6,6,6,0)
  gr.text(170,60,@oscore)
  
  repeat
    if(score_change == 1)
    
      'Clear the score_change variable
      score_change := 0

      'Display the new scores
      gr.clear
  
      gr.colorwidth(3,3)
      gr.textmode(6,6,6,0)
      pscore[0] := $30 + player_score
      gr.text(50,60,@pscore)
       
      gr.colorwidth(3,3)
      gr.textmode(6,6,6,0)
      oscore[0] := $30 + opp_score
      gr.text(170,60,@oscore)

      if(winloss == 1)
        gr.colorwidth(1,2)
        gr.textmode(4,4,6,0)
        gr.text(25,140,@vic)
      elseif(winloss == 2)
        gr.colorwidth(2,2)
        gr.textmode(4,4,6,0)
        gr.text(40,140,@def)
          

PUB MonitorButtons

  'This routine will run in another cog and monitor the gamepad.
  'It runs as a continuous loop waiting for the player to push any button.
  
  repeat
               
    'Wait for the player to stop pushing on the pad
    repeat until (pad.read & $ff == 0)

    'Debounce button
    waitcnt(cnt + $10000)

    'Wait for the player to push something on the pad
    repeat until (pad.read & $ff <> 0)

    'Increment player score
    player_score += 1

    'Determine if we have won
    if(winloss == 0 and player_score == 5)
      winloss := 1 

    'Grab the ethernet loack and send new score to opponent
    repeat while(lockset(eth_lock) == true)
    eth.TX(@player_score, 4)
    lockclr(eth_lock)

    'Set the score_change variable
    score_change := 1

    'Debounce button
    waitcnt(cnt + $10000)

PUB MonitorEthernet | UDP_Header0, UDP_Header1, data

  'This routine will run in another cog and monitor the network.
  'It runs as a continuous loop waiting for data to come from the opponent
  'over the EtherX card.

  repeat

    'Grab the ethernet lock
    repeat while(lockset(eth_lock) == true)

    'If read_rsr > 0 then there is data in the EtherX card for us.
    if(eth.read_rsr > 0)

      'We are making a dangerous assumption here that the
      'only data to have arrived is the data from the other
      'Hydra and we KNOW that it will send 1 words of data.
      'Note here however, that we read 3 words. This is because
      'when the W5100 reads UDP data the UDP header is included
      'in the data. The UDP header is 8 bytes (two longs) so we
      'read two longs of data which we junk. Then we read the data
      'we care about.
      eth.RX(@UDP_Header0, 4, false)
      eth.RX(@UDP_Header1, 4, false)
      eth.RX(@data, 4, false)

      lockclr(eth_lock)

      'Set the opponents score to the data received
      opp_score := data

      'Determine if we lost
      if(winloss == 0 and opp_score == 5)
        winloss := 2

      'Set the score_change variable
      score_change := 1

    else
      lockclr(eth_lock)
       

CON

  'Constants for UDP for TCP port
  UDP = 0
  TCP = 1
   
  'Define the port number
  PORT = 120

  'Define constant for the Ethernet initialization
  'Destination IP. The address we want to commune with
  DIP_0 = 192
  DIP_1 = 168
  DIP_2 = 1
  DIP_3 = 101

  'IP. This is our IP address
  IP_0  = 192
  IP_1  = 168
  IP_2  = 1
  IP_3  = 100

  'Gateway. This is the gateway ip address
  GWAY_0 = 192
  GWAY_1 = 168
  GWAY_2 = 1
  GWAY_3 = 1

  'Subnet
  SUB_0 = 255
  SUB_1 = 255
  SUB_2 = 255
  SUB_3 = 0

  'MAC address. This is our MAC address.
  MAC_0 = $00
  MAC_1 = $70
  MAC_2 = $6e
  MAC_3 = $69
  MAC_4 = $73
  MAC_5 = $0a        

PUB InitEther

  'This function will intialize the W5100

  'Destination IP is the IP of the Ethernet card we want to talk to
  dip[0] := DIP_0
  dip[1] := DIP_1
  dip[2] := DIP_2
  dip[3] := DIP_3

  'fill out the rest of the arrays needed for intialization
  'This is our IP address
  ip[0] := IP_0
  ip[1] := IP_1
  ip[2] := IP_2
  ip[3] := IP_3

  'Gateway
  gateway[0] := GWAY_0
  gateway[1] := GWAY_1
  gateway[2] := GWAY_2
  gateway[3] := GWAY_3

  'Subnet
  subnet[0] := SUB_0
  subnet[1] := SUB_1
  subnet[2] := SUB_2
  subnet[3] := SUB_3

  'W5100 MAC address
  mac[0] := MAC_0
  mac[1] := MAC_1
  mac[2] := MAC_2
  mac[3] := MAC_3
  mac[4] := MAC_4
  mac[5] := MAC_5

  'initialize the W5100
  eth.init(@gateway, @subnet, @ip, @mac)
  
  'open the socket
  eth.open(UDP, PORT, PORT, @dip)    
    
  
DAT

title   byte  "Button Masher",0
pushst  byte  "Push Start",0
vic     byte  "VICTORY!!",0
def     byte  "DEFEAT!!",0
pscore  byte  "0",0
oscore  byte  "0",0    
