''*****************************************************
''* This is a program that sets up the IP of the      *
''* HydraEtherX so that it can be pinged from Windows.* 
''* Written by: Shane Avery                           *
''***************************************************** 

CON

        _clkmode = xtal1 + pll8x
        _xinfreq = 10_000_000 + 0000

        ' NES bit encodings for NES gamepad 0
        NES0_RIGHT  = %00000000_00000001
        NES0_LEFT   = %00000000_00000010
        NES0_DOWN   = %00000000_00000100
        NES0_UP     = %00000000_00001000
        NES0_START  = %00000000_00010000
        NES0_SELECT = %00000000_00100000
        NES0_B      = %00000000_01000000
        NES0_A      = %00000000_10000000

VAR
        byte  subnet[4]
        byte  ip[4]
        byte  gateway[4]
        byte  mac[8]
        byte  buffer[256]                   

OBJ

        term    : "tv_terminal_010_finish.spin"     ' instantiate the terminal object
        eth     : "W5100_drv_011.spin"       ' instantiate the W5100 driver
        pad     : "gamepad_drv_001"
        gr      : "graphics_drv_010.spin"

PUB begin | ip0, ip1, ip2, ip3, whichip, padstate

  'start gamepad
  pad.start

  'Start the tv terminal.
  term.start

  'Start the W5100 driver.
  eth.start

  'Display a title string and indicate we are
  'waiting for the ethernet to come up.
  term.out($02)
  term.pstring(string("HydraEtherX Ping",13,13))
  term.out($01)
  term.pstring(string("Waiting for Eth",13))
  
  'Wait for awhile for the ethernet to come up.
  waitcnt(cnt + $17d78400)

  'Indicate that we are done waiting.
  term.pstring(string("Done waiting",13,13))

  'Wait for awhile for the user to see we are done waiting.
  waitcnt(cnt + 80_000_000)

  'Init variables
  whichip := 0
  
  'Init the IP numbers
  ip0 := 192
  ip1 := 168
  ip2 := 1
  ip3 := 100

  'Main loop for entering the IP address via the gamepad
  repeat

    'Display instructions to user.
    term.out(0)  
    term.out($02)
    term.pstring(string("HydraEtherX Ping",13,13))
    term.out($01)
    term.pstring(string("Use gamepad to enter IP",13,13))

    'Highlight which ip we are working on by making it red.
    if(whichip == 0)
      term.out($03)
      term.dec(ip0)
      term.out($01)
    else
      term.dec(ip0)
     
    term.out(46)
    if(whichip == 1)
      term.out($03)  
      term.dec(ip1)
      term.out($01)
    else
      term.dec(ip1)
     
    term.out(46)
    if(whichip == 2)
      term.out($03)  
      term.dec(ip2)
      term.out($01)
    else
      term.dec(ip2)
     
    term.out(46)
    if(whichip == 3)
      term.out($03)  
      term.dec(ip3)
      term.out($01)
    else
      term.dec(ip3)
     
    term.out(13)
    term.out(13)

    'If it is our last time through then quit the loop
    if(whichip == 4)
      quit

    'Read the pad to determine what we should do.
    padstate := 0
    repeat while padstate == 0
      padstate := pad.read & $FF

    'If the user pushes a button then we are done.
    if(padstate == NES0_A or padstate == NES0_B)
      whichip := 4
      repeat while padstate <> 0
        padstate := pad.read & $FF

    'If the user pushes right then switch to the next IP.
    if(padstate == NES0_RIGHT)
      if(whichip < 3)
        whichip += 1
      repeat while padstate <> 0
        padstate := pad.read & $FF

    'If the user pushes right then switch to the previous IP.
    if(padstate == NES0_LEFT)
      if(whichip > 0)
        whichip -= 1
      repeat while padstate <> 0
        padstate := pad.read & $FF

    'If the user pushes or holds the up button then we will
    'increment the number in that field.
    if(padstate == NES0_UP)
      if(whichip == 0)
        if(ip0 < 255)
          ip0 += 1
      if(whichip == 1)
        if(ip1 < 255)
          ip1 += 1
      if(whichip == 2)
        if(ip2 < 255)
          ip2 += 1
      if(whichip == 3)
        if(ip3 < 255)
          ip3 += 1
      waitcnt(cnt + 7_000_000)
      term.finish

    'If the user pushes or holds the down button then we will
    'decrement the number in that field.
    if(padstate == NES0_DOWN)
      if(whichip == 0)
        if(ip0 > 0)
          ip0 -= 1
      if(whichip == 1)
        if(ip1 > 0)
          ip1 -= 1
      if(whichip == 2)
        if(ip2 > 0)
          ip2 -= 1
      if(whichip == 3)
        if(ip3 > 0)
          ip3 -= 1
      waitcnt(cnt + 7_000_000)
      term.finish 

  'Fill out the arrays needed for intialization.

  'Fill out the rest of the arrays needed for intialization.
  'W5100 IP address. Use the IP address supplied by the user.
  ip[0] := ip0
  ip[1] := ip1
  ip[2] := ip2
  ip[3] := ip3

  'Subnet.
  subnet[0] := 255
  subnet[1] := 255
  subnet[2] := 255
  subnet[3] := 0

  'W5100 MAC address.
  mac[0] := $00
  mac[1] := $70
  mac[2] := $6e
  mac[3] := $69
  mac[4] := $73
  mac[5] := $0a

  'Initialize the W5100.
  eth.init(@gateway, @subnet, @ip, @mac)

  'Indicate that we can ping the device now
  term.pstring(string("Try to ping now!",13,13))

  'Wait forever
  repeat
  